unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    Button1: TButton;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
{$R *.DFM}

function CountSections(cvtFile: string): Integer;
var
  s: string;
  f: TextFile;
begin
{$I-}
  if FileExists(cvtFile) then
  try
    Result := 0;
    AssignFile(f, cvtFile);
    Reset(f);
    if IOResult = 0 then
    try
      while not eof(f) do
      begin
        Readln(f, s);
        if s <> '' then
        begin
          if s[1] = '[' then
            inc(Result);
        end;
      end;
    finally
      CloseFile(f);
    end;
  finally
  end;
{$I+}
end;

function Create_Conversion_HTML_Table(cvtFile: string): string;
var
  s, temps: string;
  f: TextFile;
  errcode, errcode2, i, j, currcount, sectioncount: Integer;
  utf8, valu: Word;
  arr: array of array[0..$FF] of record
    valu: Word;
    name: string;
  end;
  tablenames: array of string;
begin
  sectioncount := CountSections(cvtFile);
  setlength(arr, sectioncount);
  setlength(tablenames, sectioncount);
{$I-}
  AssignFile(f, cvtFile);
  Reset(f);
  if IOResult = 0 then
  try
    i := -1;
    while (i < sectioncount - 1) or (not eof(f)) do
    begin
      Readln(f, s);
      if s <> '' then
      begin
        if s[1] = '[' then
        begin
          inc(i);
          tablenames[i] := copy(s, pos('[', s) + 1, pos(']', s) - pos('[', s) - 1);
          for j := 0 to $FF do
          begin
            arr[i][j].valu := $FFFF;
            arr[i][j].name := '';
          end;
        end
        else
          if s[1] = '=' then
          begin
            temps := '$' + copy(s, 2, 2);
            val(temps, valu, errcode);
            temps := '$' + copy(s, pos('U+', s) + 2, 4);
            val(temps, utf8, errcode2);
            if (errcode or errcode2) = 0 then
              arr[i][valu].valu := utf8;
            arr[i][valu].name := copy(s, pos('U+', s) + 7, length(s));
          end;
      end;
    end;
  finally
    CloseFile(f);
  end;
{$I+}
  result := '';
  for j := -1 to $FF do
    case j of
      -1:
        begin
          result := result + '<tr>' + #13#10;
          result := result + '  <th>&nbsp;' + #13#10;
          for i := 0 to sectioncount - 1 do
          begin
            result := result + Format('  <th><span title="%s">%s</span>', [copy(tablenames[i], pos('|', tablenames[i]) + 1, length(tablenames[i])), copy(tablenames[i], 1, pos('|', tablenames[i]) - 1)]) + #13#10;
          end;
        end;
    else
      begin
        result := result + '<tr>' + #13#10;
        result := result + Format('  <td>0x%2.2X' + #13#10, [j]);
        for i := 0 to sectioncount - 1 do
          if arr[i][j].valu <> $FFFF then
          begin
            case arr[i][j].valu of
              $20:
                begin
                  result := result + Format('  <td><span title="%s (U+%4.4X)">&nbsp;</span>' + #13#10, [arr[i][j].name, arr[i][j].valu, arr[i][j].valu]);
                end;
            else
              begin
                if arr[i][j].name = '' then
                  result := result + Format('  <td><span>&#x%2.2X;</span>' + #13#10, [arr[i][j].valu])
                else
                  result := result + Format('  <td><span title="%s (U+%4.4X)">&#x%4.4X;</span>' + #13#10, [arr[i][j].name, arr[i][j].valu, arr[i][j].valu]);
              end;
            end;
          end
          else
          begin
            result := result + '  <td>&nbsp;' + #13#10;
          end;
      end;
    end;
end;

procedure TForm1.Button1Click(Sender: TObject);
const
  head =
    '<html>' + #13#10 +
    '<head>' + #13#10 +
    '<title>Code table overview for cyrillic</title>' + #13#10 +
    '<style type="text/css">' + #13#10 +
    '<!--' + #13#10 +
    'body' + #13#10 +
    '     { margin-top:13px; margin-bottom:13px; }' + #13#10 +
    'p,h1,h2,h3,h4,ol,ul,li,div,td,th,address,blockquote,nobr,b,i' + #13#10 +
    '     { font-family:Arial,Helvetica; }' + #13#10 +
    'p,ul,ol,li,div,td,th,address,blockquote,nobr,b,i' + #13#10 +
    '     { font-size:10pt; text-align:justify; alignment:justify; }' + #13#10 +
    'td,th' + #13#10 +
    '     { text-align:center; alignment:center; }' + #13#10 +
    'caption' + #13#10 +
    '     { font-weight:bold; }' + #13#10 +
    '-->' + #13#10 +
    '</style>' + #13#10 +
    '</head>' + #13#10 +
    '<body>' + #13#10 +
    '<p>Hold the mouse pointer over the respective cyrillic letter to see a description and the unicode value. Try the same for the heading line (the names of the code tables will appear in full extent).' + #13#10 +
    '<p>Note: This page is best viewed with the Mozilla rendering engine!' + #13#10 +
    '<p>' + #13#10 +
    '<table width=100% border=1>' + #13#10 +
    '<caption>Code table overview for cyrillic (UTF8, KOI8, ISO, Windows 1251)</caption>' + #13#10;
  foot =
    '</table>' + #13#10 +
    '</body>' + #13#10 +
    '</html>';

begin
  memo1.text := head + Create_Conversion_HTML_Table('ru-conversion.txt') + foot;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  memo1.Lines.SaveToFile('codetables.html');
end;

end.

